SystemOrganization addCategory: #'Athens-SVG'!

!AthensCairoPathBuilder methodsFor: '*Athens-SVG' stamp: 'IgorStasenko 4/11/2013 15:18'!
ellipticalArc: radii xrot: rotation large: largeFlag sweep: sweepFlag to: pt

	 self cwArcTo: pt angle: 45 degreesToRadians  ! !

Morph subclass: #ASVGMorph
	instanceVariableNames: 'svg surface zoom session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASVGMorph class methodsFor: 'instance creation' stamp: 'IgorStasenko 4/23/2012 16:52'!
fromFile: aFileName
	| svg |
	svg := AthensSVGConverter fromFile: aFileName.
	^ self new svg: svg ! !

!ASVGMorph methodsFor: 'session management' stamp: 'IgorStasenko 4/11/2013 14:58'!
checkSession

	session == Smalltalk session ifFalse: [
		self initializeForNewSession	
	]! !

!ASVGMorph methodsFor: 'drawing' stamp: 'IgorStasenko 4/11/2013 14:58'!
drawOn: aCanvas

	self checkSession.

	surface drawDuring: [:canvas |  
		
		surface clear.
"		canvas setPaint: Color red;
		drawShape:  (0@0 corner: self extent).
"		canvas pathTransform restoreAfter: [
			canvas pathTransform scaleBy:zoom.
			svg renderOn: canvas viewportExtent: self extent.
		]
	 ].
	aCanvas translucentImage: surface asForm at: self innerBounds origin! !

!ASVGMorph methodsFor: 'initialize' stamp: 'IgorStasenko 4/24/2012 17:28'!
initialize
	super initialize. 
	zoom :=1.! !

!ASVGMorph methodsFor: 'session management' stamp: 'IgorStasenko 4/11/2013 20:32'!
initializeForNewSession	
	| w h |
	svg width ifNotNil: [
		w := svg width valueForDPI: self screenDPI.
		h := svg height valueForDPI: self screenDPI.
		] ifNil: [
		w := 500. h := 400.
		].
	surface := AthensCairoSurface extent: (w asInteger @ (h asInteger + 1)).
	
	self extent: surface extent.
	
	session := Smalltalk session.
	
	! !

!ASVGMorph methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:37'!
screenDPI 
	^ 96 "just a guess "! !

!ASVGMorph methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:07'!
setZoom: factor

	| w h |
	w := svg width valueForDPI: self screenDPI.
	h := svg height valueForDPI: self screenDPI.
	zoom := factor.
	surface := AthensCairoSurface extent: (w*factor) asInteger @ ((h*factor) asInteger + 1).
	
	self extent: surface extent.
	self changed! !

!ASVGMorph methodsFor: 'accessing' stamp: 'IgorStasenko 4/11/2013 15:00'!
svg: aSVGRoot
	
	svg := aSVGRoot.
	
	self initializeForNewSession.
	
	self changed! !

Morph subclass: #ASZoomerMorph
	instanceVariableNames: 'basicExtent maxZoom currentAnimation currentZoom targetZoom animStartZoom svg surface svgExtent svgZoom animCenter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:52'!
animateZoomTo: zoomLevel


	targetZoom := zoomLevel.
	animStartZoom := currentZoom.

	currentAnimation := ASAnimation new subject: self.
	currentAnimation stopCondition: [:anim | anim duration >= self maxAnimationDuration ].
	currentAnimation stepAction:  [:anim | | duration |
		duration := (anim duration min: self maxAnimationDuration) / self maxAnimationDuration asFloat.
		
		self setZoom:  animStartZoom * (1-duration) + (targetZoom * duration).

		 ].
	currentAnimation finishAction: [:anim |
		self stopStepping.
		self setZoom: targetZoom.
		currentAnimation := nil.
		].
	
	currentAnimation start.
	self startStepping! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 15:14'!
drawDropShadowOn: aCanvas
! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:36'!
drawOn: aCanvas
	
	svg ifNotNil: [
		
		surface drawDuring: [:canvas |  
		
			surface clear.
			canvas pathTransform restoreAfter: [
				canvas pathTransform scaleBy: (currentZoom * svgZoom) asFloat.
				svg renderOn: canvas.
			]
		 ].
		aCanvas translucentImage: surface asForm at: self innerBounds origin
		
	] ifNil: [
		super drawOn: aCanvas.
	]! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:52'!
extent: pos

	super extent: pos.
	
	animCenter := self bounds center.
	! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 15:17'!
handlesDropShadowInHand
	"Answer whether the receiver will handle drop shadow drawing when picked up in the hand."

	^ true! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 11:11'!
handlesMouseOver: evt
	^ true! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 12:50'!
initialize 


	super initialize.
	
	currentZoom := 1.
	basicExtent  := self extent.! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:49'!
justDroppedInto: aMorph event: anEvent

	super justDroppedInto: aMorph event: anEvent.
	
	animCenter := self bounds center.! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:54'!
maxAnimationDuration
	^ 100! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:09'!
maxZoom
	
	^ maxZoom ifNil: [ maxZoom := 2 ]! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:10'!
maxZoom: aValue

	maxZoom := aValue max: 1
! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 15:04'!
mouseEnter: evt
	self comeToFront.
	self animateZoomTo: self maxZoom! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 11:14'!
mouseLeave: evt
	self animateZoomTo: 1! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:51'!
position: pos

	super position: pos.
	
	animCenter := self bounds center.
	! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 13:59'!
screenDPI 
	^ 96 "just a guess "! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:50'!
setZoom: aZoom
	|  newExtent oldCenter |
	currentZoom := aZoom.
	newExtent := basicExtent * aZoom.
	
	oldCenter := animCenter.
	self bounds: (animCenter - (newExtent /2) extent: newExtent).
	animCenter := oldCenter.
	
	self changed! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 11:44'!
step

	currentAnimation ifNotNil: [
		currentAnimation step
		]! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 12:01'!
stepTime
	^ 20! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:28'!
svg: anSVG

	^ self svg: anSVG zoom: 1! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:37'!
svg: anSVG zoom: anSvgZoom
	| w h |
	
	svg := anSVG.
	svg isString ifTrue: [ svg :=  AthensSVGConverter fromFile: svg ].
	svgExtent := self extent.
	
	svg width ifNotNil: [
		w := svg width valueForDPI: self screenDPI.
		h := svg height valueForDPI: self screenDPI.
		svgExtent := (w@h) * anSvgZoom + (2@2).
		svgZoom := anSvgZoom.
	] ifNil: [
		svgExtent := self extent.
		svgZoom := 1.
	].

	surface := AthensCairoSurface extent: (svgExtent * self maxZoom + (1@1)) asIntegerPoint.
	basicExtent := svgExtent.
	self extent: svgExtent.
	currentZoom := 1.	
	self changed! !

!ASZoomerMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/10/2012 14:48'!
wantsToBeDroppedInto: aMorph

	^ aMorph isWorldMorph! !

Object subclass: #ASAnimation
	instanceVariableNames: 'subject finishAction stepAction started stopCondition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:29'!
duration 
	^ Time millisecondClockValue - started! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:15'!
finishAction

	^ finishAction! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:15'!
finishAction: anObject

	finishAction := anObject! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:28'!
start

	started := Time millisecondClockValue.! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:34'!
step

	(stopCondition value: self )
		ifTrue: [ finishAction value: self ] 
		ifFalse: [ stepAction value: self  ]! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:15'!
stepAction

	^ stepAction! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:29'!
stepAction: aBlock

	stepAction := aBlock! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:30'!
stopCondition: aBlock

	stopCondition := aBlock! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:15'!
subject

	^ subject! !

!ASAnimation methodsFor: 'accessing' stamp: 'IgorStasenko 6/10/2012 11:15'!
subject: anObject

	subject := anObject! !

Object subclass: #ASConverter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASConverter commentStamp: 'StephaneDucasse 12/6/2014 07:12' prior: 0!
A converter is a kind of parser.!

!ASConverter methodsFor: 'parsing-public' stamp: 'IgorStasenko 4/27/2012 16:17'!
nextNumber
	"read the number from stream.
	or answer nil if not successfull"
	
"
<number> (real number value): The specification of real number values is different for property values than for XML attribute values.
❍ CSS2 [CSS2] states that a property value which is a <number> is specified in decimal notation (i.e., a <decimal-number>), which consists of either an <integer>, or an optional sign character followed by
zero or more digits followed by a dot (.) followed by one or more digits. Thus, for conformance with
CSS2, any property in SVG which accepts <number> values is specified in decimal notation only.
❍ For SVG's XML attributes, to provide as much scalability in numeric values as possible, real number
values can be provided either in decimal notation or in scientific notation (i.e., a <scientific-number>), which consists of a <decimal-number> immediately followed by the letter ''e'' or ''E'' immediately followed by an <integer>.
Unless stated otherwise for a particular attribute or property, a <number> has the capacity for at least a single- precision floating point number (see [ICC32]) and has a range (at a minimum) of -3.4e+38F to +3.4e+38F.
It is recommended that higher precision floating point storage and computation be performed on operations such as coordinate system transformations to provide the best possible precision and to prevent round-off errors.
Conforming High-Quality SVG Viewers are required to use at least double-precision floating point (see [ICC32]) for intermediate calculations on certain numerical operations.
Within the SVG DOM, a <number> is represented as a float or an SVGAnimatedNumber.
"
	
	| result char sign esign pos int frac exponent |
	
	esign := sign := 1.
	pos := stream position.
	
	stream peek = $- ifTrue: [ sign := -1.  stream next ].

	int := self readInteger.
	
	stream peek = $. ifTrue: [
		stream next.
		frac := self readFraction.
	].
	
	(stream peek = $e or: [ stream peek = $E ]) ifTrue: [
		stream next.
		stream peek = $- ifTrue: [ esign := -1.  stream next ].

		exponent := self readInteger * esign ].
	
	"failed"
	(int isNil and: [ frac isNil ]) ifTrue: [ stream position: pos.  ^ nil ].
	
	int ifNil: [ int := 0 ].
	frac ifNil: [ frac := 0 ].
	exponent ifNil: [ exponent := 0 ].
	
	result :=  (int + frac * (10 raisedTo: exponent) * sign).
	
	result isFraction ifTrue: [  ^ result asFloat ] ifFalse: [ ^ result ]
! !

!ASConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/27/2012 11:50'!
noFill
	^ ASNoFill soleInstance ! !

!ASConverter methodsFor: 'parsing' stamp: 'IgorStasenko 4/24/2012 16:40'!
parseColor: aString

	^ Color fromString: aString! !

!ASConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/24/2012 19:11'!
readFraction
"
read fraction, what is after the decimal point:

.12345


"
	| result mul |
	(stream atEnd or: [ stream peek isDigit not ]) ifTrue: [ ^ nil ].
	
	mul := 1/10. 
	result := 0.
	[ stream peek notNil and: [ stream peek isDigit ] ] whileTrue: [
		result := result + ( (stream next digitValue) * mul ).
		mul := mul / 10
	].
	^ result! !

!ASConverter methodsFor: 'write and read' stamp: 'IgorStasenko 4/24/2012 16:07'!
readInteger
	| result |
	(stream atEnd or: [ stream peek isDigit not ]) ifTrue: [ ^ nil ].
	
	result := 0.
	[ stream peek notNil and: [ stream peek isDigit ] ] whileTrue: [
		result := result * 10 + (stream next digitValue)
	].
	^ result! !

!ASConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/23/2012 15:48'!
skipBlanks
	[ stream atEnd not and: [ stream peek isSeparator ] ] whileTrue: [ stream next ].! !

!ASConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/25/2012 16:30'!
skipBlanksAndComma
	[ stream atEnd not and: [ stream peek isSeparator or: [stream peek =$,]] ] whileTrue: [ stream next ].! !

!ASConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 15:58'!
stream
	^ stream! !

!ASConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 15:49'!
stream: aStream
	stream := aStream! !

ASConverter subclass: #ASFillConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASFillConverter methodsFor: 'translate' stamp: 'AliakseiSyrel 7/30/2015 12:12'!
translate: aString

	 
	(aString first = $#) ifTrue: [
		"color"
		^ ASColorPaint new color: (self parseColor: (aString allButFirst)).
	].

	aString = 'none' ifTrue: [  ^ self noFill ].
	
	(aString beginsWith: 'url') ifTrue: [
		"ok, we got a link"
		| str |
		
		str := (aString copyAfter: $( ) copyUpTo: $).
		^ ASLink new url: str.

		].
	
	(aString beginsWith: 'rgb') ifTrue: [
		| str numbers |
		str := (aString copyAfter: $( ) copyUpTo: $).
		numbers := (str substrings: ',') collect: [:ea| ea trimBoth asNumber].

		^ ASColorPaint new color: (
			Color r: (numbers first/255)
				g: (numbers second/255)
				b: (numbers third/255)
			).

		].
	"can be just a color name"
	(Color colorNames includes: aString) ifTrue: [
		^ ASColorPaint new color: (Color perform: aString asSymbol)
		].
	
	
	self halt.
	! !

ASConverter subclass: #ASPathConverter
	instanceVariableNames: 'segments path absolute min max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASPathConverter commentStamp: '<historical>' prior: 0!
The syntax of path data is concise in order to allow for minimal file size and efficient downloads, since many SVG files will be dominated by their path data. Some of the ways that SVG attempts to minimize the size of path data are as follows:
● All instructions are expressed as one character (e.g., a moveto is expressed as an M).
● Superfluous white space and separators such as commas can be eliminated (e.g., "M 100 100 L 200 200"
contains unnecessary spaces and could be expressed more compactly as "M100 100L200 200").
● The command letter can be eliminated on subsequent commands if the same command is used multiple times
in a row (e.g., you can drop the second "L" in "M 100 200 L 200 100 L -100 -200" and use "M 100 200 L 200
100 -100 -200" instead).
● Relative versions of all commands are available (uppercase means absolute coordinates, lowercase means
relative coordinates).
● Alternate forms of lineto are available to optimize the special cases of horizontal and vertical lines (absolute and
relative).
● Alternate forms of curve are available to optimize the special cases where some of the control points on the
current segment can be determined automatically from the control points on the previous segment.!

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/25/2012 17:06'!
A
	self absolute; ellipticalArc! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/23/2012 19:16'!
C

	^ self absolute; cubicBezier ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:15'!
H
	self absolute; hLineTo ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/23/2012 19:21'!
L
	self absolute; lineTo.! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/23/2012 19:17'!
M
	"move-to absolute"
	" 
		Start a new sub-path at the given (x,y) coordinate. M (uppercase) indicates that absolute coordinates will follow; m (lowercase) indicates that relative coordinates will follow. 
		If a relative moveto (m) appears as the first element of the path, then it is treated as a pair of absolute coordinates. 
		If a moveto is followed by multiple pairs of coordinates, the subsequent pairs are treated as implicit lineto commands.
	
	"
	self absolute; moveTo.
! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:20'!
Q
	self absolute; quadraticBezier! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:18'!
S
	self absolute; reflectedCubicBezier! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:20'!
T
	self absolute; smoothQuadraticBezier! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:15'!
V
	self absolute; vLineTo ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:09'!
Z
	self close! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/25/2012 17:06'!
a
	self relative; ellipticalArc! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/25/2012 08:38'!
absolute
	absolute := true.
	self addSegment ! !

!ASPathConverter methodsFor: 'adding' stamp: 'IgorStasenko 4/25/2012 08:41'!
addSegment

	| sel args |
	sel := 	thisContext sender selector.
	args := thisContext sender arguments.
	"update min/max for calculatin bounding box"
	
	path add: { sel. args }! !

!ASPathConverter methodsFor: 'converting' stamp: 'IgorStasenko 4/23/2012 18:29'!
assertValidCommand: aCharacter 

	( #( $M $m $Z $z $L $l $H $h $V $v $C $c $S $s $Q $q $T $t $A $a ) includes: aCharacter ) ifFalse: [
		self error: 'invalid path segment command' ]
! !

!ASPathConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 08:42'!
bbox
	^ 100@100! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/23/2012 19:16'!
c

	^ self relative; cubicBezier ! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
close
	self addSegment ! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/23/2012 19:26'!
convertPathData

	path := OrderedCollection new.
	
	[ self skipBlanks. stream atEnd ] whileFalse: [
		| cmd |
		cmd := stream next.
		self assertValidCommand: cmd.
		self perform: cmd asString asSymbol.
	].

	^ path
	! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/24/2012 18:14'!
cubicBezier


"
C/c (x1 y1 x2 y2 x y)+

Draws a cubic Bézier curve from the current point to (x,y) using (x1,y1) as the control point at the beginning of the curve and (x2,y2) as the control point at the end of the curve. 
C (uppercase) indicates that absolute coordinates will follow; c (lowercase) indicates that relative coordinates will follow. Multiple sets of coordinates may be specified to draw a polybézier. 
At the end of the command, the new current point becomes the final (x,y) coordinate pair used in the polybézier.
"
	| p1 p2 p3 |
	
	p1 := self readPoint.
	p2 := self readPoint.
	p3 := self readPoint.
	
	p3 ifNil: [ self error: 'Invalid path data: 3 points expected for cubic bezier '].

	self curveVia: p1 and:  p2 to: p3. 
	
	[ 
		(p1 := self readPoint) ifNil: [ ^ self ].
		p2 := self readPoint.
		p3 := self readPoint.

		p3 ifNil: [ self error: 'Invalid path data: 3 points expected for cubic bezier '].
		self curveVia: p1 and:  p2 to: p3. 
	] repeat.

! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
curveVia: aPoint and: aPoint2 to: aPoint3 

	self addSegment ! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
curveVia: aPoint to: aPoint2 


	self addSegment ! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/25/2012 21:12'!
ellipticalArc 


"
A (absolute) a (relative)

elliptical arc
(rx ry x-axis-rotation large-arc-flag sweep-flag x y)+
Draws an elliptical arc from the current point to (x, y). The size and orientation of the ellipse are defined by two radii (rx, ry) and an x-axis-rotation, which indicates how the ellipse as a whole is rotated relative to the current coordinate system. The center (cx, cy) of the ellipse is calculated automatically to satisfy the constraints imposed by the other parameters. large-arc-flag and sweep- flag contribute to the automatic calculations and help determine how the arc is drawn.
"
	| r xrot large sweep pt |
	
	[
	r := self readPoint.
	r ifNil: [ ^ self ].
	
	self skipBlanksAndComma.
	
	xrot := self nextNumber.

	self skipBlanksAndComma.
	
	large := self nextNumber.

	self skipBlanksAndComma.

	sweep := self nextNumber.
	
	pt := self readPoint.
	
	pt ifNil: [ self error: 'Invalid path data for elliptical arc segment'].

	self ellipticalArc: r xrot: xrot large: large sweep: sweep to: pt.
	] repeat.

! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/11/2013 15:14'!
ellipticalArc: r xrot: xrot large: large sweep: sweep to: pt

	self addSegment! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:15'!
h
	self relative; hLineTo ! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/24/2012 16:14'!
hLineTo
"
H (absolute) h (relative)

x +
Draws a horizontal line from the current point (cpx, cpy) to (x, cpy). H (uppercase) indicates that absolute coordinates will follow; h (lowercase) indicates that relative coordinates will follow. Multiple x values can be provided (although usually this doesn't make sense). At the end of the command, the new current point becomes (x, cpy) for the final value of x.
"

	| x |
	
	self skipBlanks.
	x := self nextNumber ifNil: [ ^ self error: 'number expected for horizontal line segment'   ].
	
	self hLineTo: x.
	
	[ self skipBlanks. x := self nextNumber. x notNil ] whileTrue: [
		self hLineTo: x
	].
! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
hLineTo: aNumber

	self addSegment ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:09'!
l
	self relative; lineTo.! !

!ASPathConverter methodsFor: 'operations' stamp: 'JochenRick 1/8/2014 06:01'!
lastBezierPoint
	path reverseDo: [ :arr | arr first caseOf: { 
		[ #relative ] -> [ "ignore" ] } ]! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/23/2012 19:23'!
lineTo
" 
L (absolute) l (relative)
lineto

(x y)+
Draw a line from the current point to the given (x,y) coordinate which becomes the new current point. L (uppercase) indicates that absolute coordinates will follow; l (lowercase) indicates that relative coordinates will follow. A number of coordinates pairs may be specified to draw a polyline. At the end of the command, the new current point is set to the final set of coordinates provided.
"

	| pt |
	pt := self readPoint.
	
	pt ifNil: [ self error: 'point expected for line segment' ].
	
	self lineTo: pt.
	
	[ pt := self readPoint. pt notNil ] whileTrue: [ self lineTo: pt ]! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
lineTo: aPoint 

	self addSegment ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/27/2012 14:20'!
m
	"move-to relative"
	" 
		Start a new sub-path at the given (x,y) coordinate. M (uppercase) indicates that absolute coordinates will follow; m (lowercase) indicates that relative coordinates will follow. 
		If a relative moveto (m) appears as the first element of the path, then it is treated as a pair of absolute coordinates. 
		If a moveto is followed by multiple pairs of coordinates, the subsequent pairs are treated as implicit lineto commands.
	
	"
	| pt reset |

	pt := self readPoint.
	pt ifNil: [ self error: 'a point expected following after M/m command'].

	path isEmpty ifTrue: [ 
			self absolute; moveTo: pt; relative ] 
		ifFalse: [ self relative; moveTo: pt ].

	[ pt := self readPoint. pt notNil ] whileTrue: [ self lineTo: pt ]! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/23/2012 19:17'!
moveTo
	"move-to"
	" 
		Start a new sub-path at the given (x,y) coordinate. M (uppercase) indicates that absolute coordinates will follow; m (lowercase) indicates that relative coordinates will follow. 
		If a relative moveto (m) appears as the first element of the path, then it is treated as a pair of absolute coordinates. 
		If a moveto is followed by multiple pairs of coordinates, the subsequent pairs are treated as implicit lineto commands.
	
	"
	| pt |

	pt := self readPoint.
	pt ifNil: [ self error: 'a point expected following after M/m command'].
	
	self moveTo: pt.
	
	[ pt := self readPoint. pt notNil ] whileTrue: [ self lineTo: pt ]
! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
moveTo: aPoint 

	self addSegment ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:20'!
q
	self relative; quadraticBezier! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/24/2012 16:28'!
quadraticBezier 


"
Q/q (x1 y1 x y)+

Draws a quadratic Bézier curve from the current point to (x,y) using (x1,y1) as the control point. Q (uppercase) indicates that absolute coordinates will follow; q (lowercase) indicates that relative coordinates will follow. Multiple sets of coordinates may be specified to draw a polybézier. At the end of the command, the new current point becomes the final (x,y) coordinate pair used in the polybézier."
	| p1 p2 |
	
	p1 := self readPoint.
	p2 := self readPoint.
	
	p2 ifNil: [ self error: 'Invalid path data: 2 points expected for quadratic bezier '].
	
	self curveVia: p1 to: p2. 
	
	[ 
		(p1 := self readPoint) ifNil: [ ^ self ].
		p2 := self readPoint.

		p2 ifNil: [ self error: 'Invalid path data: 2 points expected for quadratic bezier '].
		self curveVia: p1 to: p2. 
	] repeat.

! !

!ASPathConverter methodsFor: 'operations' stamp: 'JochenRick 1/6/2014 09:38'!
readPoint
	" A path data permits specifying points without comma separator "
	| x y pos |
	self skipBlanksAndComma.
	pos := stream position.
	x := self nextNumber ifNil: [ ^ nil ].
	
	self skipBlanksAndComma.
	
	y := self nextNumber ifNil: [ stream position: pos. ^ nil ].
	
	^ x @ y! !

!ASPathConverter methodsFor: 'operations' stamp: 'JochenRick 1/8/2014 06:19'!
reflectedCubicBezier

"
S/s (x2 y2 x y)+

cubic Bézier curve from the current point to (x,y). The first control point is assumed to be the reflection of the second control point on the previous command relative to the current point. (If there is no previous command or if the previous command was not an C, c, S or s, assume the first control point is coincident with the current point.) (x2,y2) is the second control point (i.e., the control point at the end of the curve). S (uppercase) indicates that absolute coordinates will follow; s (lowercase) indicates that relative coordinates will follow. Multiple sets of coordinates may be specified to draw a polybézier. At the end of the command, the new current point becomes the final (x,y) coordinate pair used in the polybézier.
"
	| p1 p2 |
	
	p1 := self readPoint.
	p2 := self readPoint.
	
	p2 ifNil: [ self error: 'Invalid path data: 2 points expected for reflected cubic bezier '].

	self reflectedCurveVia: p1 to: p2. 
	
	[ 
		(p1 := self readPoint) ifNil: [ ^ self ].
		p2 := self readPoint.

		p2 ifNil: [ self error: 'Invalid path data: 2 points expected for reflected cubic bezier '].
		self reflectedCurveVia: p1 to: p2. 
	] repeat.

! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'JochenRick 1/8/2014 06:11'!
reflectedCurveVia: aPoint to: aPoint2 


	self addSegment ! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/25/2012 08:38'!
relative
	absolute := false.
	self addSegment ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:18'!
s
	self relative; reflectedCubicBezier! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/24/2012 16:20'!
smoothQuadraticBezier
	self notYetImplemented ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:20'!
t
	self relative; smoothQuadraticBezier! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/24/2012 16:15'!
v
	self relative; vLineTo ! !

!ASPathConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/24/2012 16:15'!
vLineTo
"
V (absolute) v (relative)

Draws a vertical line from the current point (cpx, cpy) to (cpx, y). V (uppercase) indicates that absolute coordinates will follow; v (lowercase) indicates that relative coordinates will follow. Multiple y values can be provided (although usually this doesn't make sense). At the end of the command, the new current point becomes (cpx, y) for the final value of y."

	| y |
	
	self skipBlanks.
	y := self nextNumber ifNil: [ ^ self error: 'number expected for horizontal line segment'   ].
	
	self vLineTo: y.
	
	[ self skipBlanks. y := self nextNumber. y notNil ] whileTrue: [
		self vLineTo: y
	].
! !

!ASPathConverter methodsFor: 'builder commands' stamp: 'IgorStasenko 4/24/2012 16:32'!
vLineTo: aNumber
	
	self addSegment ! !

!ASPathConverter methodsFor: 'path commands' stamp: 'IgorStasenko 4/23/2012 19:24'!
z
	self close! !

ASConverter subclass: #ASTransformConverter
	instanceVariableNames: 'transform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASTransformConverter methodsFor: 'operations' stamp: 'JochenRick 1/24/2014 10:41'!
matrix
	"matrix(<a> <b> <c> <d> <e> <f>), which specifies a transformation in the form of a transformation matrix of six values. matrix(a,b,c,d,e,f) is equivalent to applying the transformation matrix [a b c d e f]."
	
	| m |

	m := AthensAffineTransform new.
"
Athens matrix: 

| sx   shx   x   |
| shy sy     y   |
|  0     0     1   | "	

"
SVG matrix: 
| a   c   e   |
| b   d  f  |
|  0     0     1   | "	
	

"a"	self skipBlanks.
	m sx: self nextNumber.

"b"	self skipBlanksAndComma.
	m shy: self nextNumber.

"c"	self skipBlanksAndComma.
	m shx: self nextNumber.

"d"	self skipBlanksAndComma.
	m sy: self nextNumber.

"e"	self skipBlanksAndComma.
	m x: self nextNumber.
	
"f"	self skipBlanksAndComma.
	m y: self nextNumber.

	transform multiplyBy: m
	! !

!ASTransformConverter methodsFor: 'operations' stamp: 'IgorStasenko 4/27/2012 14:02'!
rotate
	"
rotate(<rotate-angle> [<cx> <cy>]), which specifies a rotation by <rotate-angle> degrees about a given point.
If optional parameters <cx> and <cy> are not supplied, the rotate is about the origin of the current user coordinate system. The operation corresponds to the matrix [cos(a) sin(a) -sin(a) cos(a) 0 0].
If optional parameters <cx> and <cy> are supplied, the rotate is about the point (<cx>, <cy>). The operation represents the equivalent of the following specification: translate(<cx>, <cy>) rotate(<rotate- angle>) translate(-<cx>, -<cy>).
	
	"
	| angle cx cy |
	
	angle := self nextNumber.
	cx := self nextNumber.
	cy := self nextNumber.

	cx ifNil: [
		transform rotateByDegrees: angle negated .		
	] ifNotNil: [
		"if cx is given, cy should also be present"
		self assert: cy notNil. 
	
		transform translateX: cx Y: cy. 
		transform rotateByDegrees: angle negated.		
		transform translateX: cx negated  Y: cy negated. 
	].

! !

!ASTransformConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/7/2012 16:19'!
scale
	"
	scale(<sx> [<sy>]), which specifies a scale operation by sx and sy. If <sy> is not provided, it is assumed
to be equal to <sx>.
	
	"
	
	| sx sy |
	
	sx := self nextNumber.
	self skipBlanks.
	sy := self nextNumber ifNil: [ sx ].
	
	transform scaleX: sx Y: sy! !

!ASTransformConverter methodsFor: 'public access' stamp: 'IgorStasenko 4/27/2012 13:28'!
translate
	"
	translate(<tx> [<ty>]), which specifies a translation by tx and ty. If <ty> is not provided, it is assumed to be zero.
	"
	| tx ty |

	tx := self nextNumber.
	self skipBlanks.
	ty := 0.
	(stream peek = $,) ifTrue: [
		stream next.
		self skipBlanks.
		ty := self nextNumber ifNil: [ 0 ]].
	
	transform translateX: tx Y: ty! !

!ASTransformConverter methodsFor: 'operations' stamp: 'JochenRick 1/24/2014 10:28'!
translateString: aString 
	"translate the transform data 
	
	'rotate(10) scale (1) translate (100,10)'
	"

	stream := aString readStream.
	transform := AthensAffineTransform new.
		
	[ stream atEnd ] whileFalse: [ | type |
		type := (stream upTo: $() trimBoth.	
		self perform: type asSymbol.
		
		stream upTo: $).
		self skipBlanks
		 ].
	^ transform! !

Object subclass: #ASElement
	instanceVariableNames: 'parent id attributes'
	classVariableNames: 'EmptyDict'
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASElement commentStamp: 'StephaneDucasse 12/6/2014 07:18' prior: 0!
Root ot the SVG Elements

An SVG Element has 
	- attributes		
	- id		
	- parent

Pay attention that some methods contain _ because of an automatic conversion.
Read 
	attributeSetter: aString
		"fill-opacity   =>>  fill_opacity:
		x =>> x:
		"!

ASElement subclass: #ASContainerElement
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASContainerElement commentStamp: 'StephaneDucasse 12/6/2014 07:27' prior: 0!
An ASContainerElement is ASElement with children.

Instance Variables
	children!

!ASContainerElement methodsFor: 'rendering' stamp: 'IgorStasenko 4/25/2012 05:50'!
renderShapeOn: aCanvas

	"render on Athens canvas"
	children do: [:child | child renderOn: aCanvas ]! !

!ASContainerElement methodsFor: 'internal' stamp: 'StephaneDucasse 12/6/2014 07:27'!
resolveIds: idDict
	self resolveMyIds: idDict.
	children do: [ :each | each resolveIds: idDict ]! !

!ASContainerElement methodsFor: 'translate' stamp: 'IgorStasenko 4/25/2012 06:40'!
translate: node
	super translate: node.
	
	children := (node nodesCollect: [:child |
		| nodeClass |
		nodeClass := ASVGNodeClasses fromName: child name.
	
		nodeClass new parent: self; translate: child; yourself	 ]) asArray! !

ASContainerElement subclass: #ASDefs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASDefs methodsFor: 'rendering' stamp: 'IgorStasenko 4/25/2012 05:19'!
renderOn: aCanvas
	"defs are not rendered directly"! !

!ASDefs methodsFor: 'rendering' stamp: 'IgorStasenko 4/25/2012 05:50'!
renderShapeOn: aCanvas

	"defs are not rendered directly"! !

ASContainerElement subclass: #ASGradient
	instanceVariableNames: 'stops gradientTransform gradientUnits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASGradient methodsFor: 'operations' stamp: 'IgorStasenko 4/27/2012 15:02'!
colorRampWithOpacity: aFloat 
	
	(stops isNil or: [stops  isEmpty ]) ifTrue: [
		self xlink_href ifNotNil: [:link |
			^ link colorRampWithOpacity: aFloat
			].
		^ #().
		].
	
	^ stops collect: [:ea | ea asRampColorWithOpacity: aFloat]
! !

!ASGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:37'!
gradientTransform: aString

	gradientTransform := ASTransformConverter new translateString: aString! !

!ASGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:37'!
gradientUnits: aString 

	gradientUnits := aString! !

!ASGradient methodsFor: 'initialize' stamp: 'IgorStasenko 6/10/2012 11:02'!
initialize 
	super initialize.
	gradientTransform := AthensAffineTransform new.! !

!ASGradient methodsFor: 'rendering' stamp: 'IgorStasenko 4/27/2012 12:35'!
renderOn: aCanvas
	"do nothing, i'm cannot be rendered directly"! !

!ASGradient methodsFor: 'operations' stamp: 'IgorStasenko 4/27/2012 15:56'!
spreadMethod: aByteString 
	"not yet"
! !

ASGradient subclass: #ASLinearGradient
	instanceVariableNames: 'x1 y1 x2 y2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASLinearGradient methodsFor: 'converting' stamp: 'IgorStasenko 4/24/2012 17:18'!
asAthensPaintOn: anAthensCairoCanvas 
	^ self! !

!ASLinearGradient methodsFor: 'operations' stamp: 'IgorStasenko 4/21/2013 14:57'!
fill: aShape on: aCanvas

	| ramp grad p1 p2 fo |
	aCanvas paintTransform restoreAfter: [
		ramp := self colorRampWithOpacity: aShape fill_opacity * aShape opacity. 
		p1 := gradientTransform transform: x1@y1.
		p2 := gradientTransform transform: x2@y2.
		
		grad := aCanvas surface createLinearGradient:  ramp start: p1  stop: p2.
		
		aCanvas setPaint: grad.
		aCanvas draw
	].! !

!ASLinearGradient methodsFor: 'initialize' stamp: 'IgorStasenko 4/24/2012 18:10'!
initialize 
	super initialize.
	gradientTransform := AthensAffineTransform new.! !

!ASLinearGradient methodsFor: 'operations' stamp: 'NicolaiHess 4/28/2015 16:46'!
stroke: aShape on: aCanvas

	| ramp grad p1 p2 |
	aCanvas paintTransform restoreAfter: [
		ramp := self colorRampWithOpacity: aShape stroke_opacity * aShape opacity. 

		p1 := gradientTransform transform: x1@y1.
		p2 := gradientTransform transform: x2@y2.
		
		grad := aCanvas surface createLinearGradient:  ramp start: p1  stop: p2.
		
		aCanvas setStrokePaint: grad.
		aCanvas draw
	].! !

!ASLinearGradient methodsFor: 'translate' stamp: 'IgorStasenko 4/27/2012 10:48'!
translate: node
	super translate: node.
	stops := children ! !

!ASLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 13:58'!
x1: aString 

	x1 := self translateNumber: aString! !

!ASLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:49'!
x2: aString 

	x2 := self translateNumber: aString! !

!ASLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:48'!
y1: aString 

	y1 := self translateNumber: aString! !

!ASLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:50'!
y2: aString 

	y2 := self translateNumber: aString
! !

ASGradient subclass: #ASRadialGradient
	instanceVariableNames: 'cx cy r fx fy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:32'!
cx: aString 

	cx := self translateLength: aString! !

!ASRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:32'!
cy: aString 

	cy := self translateLength: aString! !

!ASRadialGradient methodsFor: 'operations' stamp: 'IgorStasenko 4/27/2012 15:27'!
fill: shape on: aCanvas
	
	| ramp grad p1 p2 |
	aCanvas paintTransform restoreAfter: [
		ramp := self colorRampWithOpacity: shape fill_opacity * shape opacity. 

		p1 := gradientTransform transform: cx@cy.
		
		fx ifNil: [ fx := cx ].
		fy ifNil: [ fy := cy ].
			
		p2 := gradientTransform transform: fx@fy.
		
		grad := aCanvas surface createRadialGradient: ramp center: p1 radius: r focalPoint: p2.
		
		aCanvas setPaint: grad.
		aCanvas draw
	].! !

!ASRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:34'!
fx: aString 
	fx := self translateLength: aString! !

!ASRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:34'!
fy: aString 
	fy := self translateLength: aString! !

!ASRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:34'!
r: aString 
	r := self translateLength: aString! !

!ASRadialGradient methodsFor: 'operations' stamp: 'IgorStasenko 4/27/2012 15:00'!
stroke: shape on: aCanvas
	
	| ramp grad p1 p2 |
	aCanvas paintTransform restoreAfter: [
		ramp := self colorRampWithOpacity: shape stroke_opacity * shape opacity. 

		p1 := gradientTransform transform: cx@cy.
		
		fx ifNil: [ fx := cx ].
		fy ifNil: [ fy := cy ].
			
		p2 := gradientTransform transform: fx@fy.
		
		grad := aCanvas surface createRadialGradient: ramp center: p1 radius: r focalPoint: p2.
		
		aCanvas setStrokePaint: grad.
		aCanvas draw
	].! !

ASContainerElement subclass: #ASGroupElement
	instanceVariableNames: 'stroke_opacity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASGroupElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 06:25'!
stroke_opacity: aString 
	stroke_opacity := self translateNumber: aString
! !

ASContainerElement subclass: #ASVGRoot
	instanceVariableNames: 'width height ids viewBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASVGRoot commentStamp: '<historical>' prior: 0!
i am root element of SVG document, correspond to a <svg> tag in document.!

!ASVGRoot methodsFor: 'to be done' stamp: 'IgorStasenko 4/23/2012 17:34'!
baseProfile: aString 
	self yourself! !

!ASVGRoot methodsFor: 'to be done' stamp: 'IgorStasenko 4/23/2012 17:31'!
contentScriptType: aString 

	"ignore"! !

!ASVGRoot methodsFor: 'to be done' stamp: 'IgorStasenko 4/23/2012 17:34'!
contentStyleType: aString 

	self yourself ! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:53'!
height
	^ height! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:26'!
height: aString 
	height := ASLength fromString: aString! !

!ASVGRoot methodsFor: 'attribute management' stamp: 'IgorStasenko 4/25/2012 22:12'!
inheritedAttributeAt: name
	"i am root, nobody to inherit from"
	^ attributes at: name ifAbsent: nil! !

!ASVGRoot methodsFor: 'initialize' stamp: 'JochenRick 1/24/2014 11:00'!
initialize 
	super initialize.
	ids := Dictionary new.! !

!ASVGRoot methodsFor: 'to be done' stamp: 'IgorStasenko 4/23/2012 17:35'!
preserveAspectRatio: aString 

	self yourself ! !

!ASVGRoot methodsFor: 'rendering' stamp: 'IgorStasenko 4/25/2012 07:40'!
renderOn: aCanvas viewportExtent: extent

	viewBox ifNil: [ ^self renderOn: aCanvas ].
	
	aCanvas pathTransform restoreAfter: [
		aCanvas pathTransform
			translateX: viewBox origin x negated Y: viewBox origin y negated;
			scaleX: (extent x / viewBox width) asFloat Y: (extent y / viewBox height) asFloat.
		self renderOn: aCanvas.		
	]! !

!ASVGRoot methodsFor: 'internal' stamp: 'IgorStasenko 4/24/2012 16:34'!
resolveIds 
	^ self resolveIds: ids! !

!ASVGRoot methodsFor: 'internal' stamp: 'IgorStasenko 4/24/2012 15:05'!
setId: anId for: object

	ids at: anId put: object! !

!ASVGRoot methodsFor: 'translate' stamp: 'IgorStasenko 4/25/2012 07:01'!
translate: node
	super translate: node.
	self resolveIds.
! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 17:34'!
version: aString 

		self yourself ! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 06:09'!
viewBox: aString 
	"min-x min-y width height"
	
	| conv mx my w h |
	
	conv := ASConverter new stream: aString readStream.
	
	mx := conv skipBlanks nextNumber.
	my := conv skipBlanks nextNumber.
	w := conv skipBlanks nextNumber.
	h := conv skipBlanks nextNumber.

	viewBox := Rectangle origin: mx@my extent: w@h! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:53'!
width
	^ width! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:26'!
width: aString
	
	width := ASLength fromString: aString
! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 17:35'!
x: aString 
	self yourself ! !

!ASVGRoot methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 17:36'!
y: aString 
	self yourself ! !

!ASVGRoot methodsFor: 'internal' stamp: 'IgorStasenko 4/23/2012 17:32'!
zoomAndPan: aString 
	"self shouldBeImplemented."! !

!ASElement class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/7/2012 14:29'!
convertXML: aXMLElement 
	^ self new convertXML: aXMLElement! !

!ASElement class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 06:28'!
initialize
	EmptyDict := IdentityDictionary new.! !

ASElement initialize!

!ASElement class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/7/2012 14:37'!
translateNode: xmlNode
	| nodeClass |
	nodeClass := ASVGNodeClasses fromName: xmlNode name.
	^ nodeClass new translate: xmlNode! !

!ASElement class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 15:03'!
translateNode: xmlNode parent: aParent
	| nodeClass |
	nodeClass := ASVGNodeClasses fromName: xmlNode name.
	^ nodeClass new translate: xmlNode parent: aParent! !

!ASElement methodsFor: 'attribute management' stamp: 'StephaneDucasse 12/6/2014 07:18'!
attributeAt: name
	^ attributes at: name ifAbsent: nil! !

!ASElement methodsFor: 'attribute management' stamp: 'StephaneDucasse 12/6/2014 07:18'!
attributeAt: name put: value
	attributes == EmptyDict
		ifTrue: [ attributes := IdentityDictionary new ].
	^ attributes at: name put: value! !

!ASElement methodsFor: 'operations' stamp: 'StephaneDucasse 12/6/2014 07:18'!
attributeSetter: aString
	"fill-opacity   =>>  fill_opacity:
	x =>> x:"
	| sel |
	sel := aString collect: [ :c | ('-:' includes: c) ifTrue: [ $_] ifFalse: [ c ]].
	^ (sel copyWith: $:) asSymbol ! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 17:09'!
color: aByteString 
	self attributeAt: #color put: aByteString ! !

!ASElement methodsFor: 'attribute management' stamp: 'StephaneDucasse 12/6/2014 07:18'!
defaultAttributeAt: aName
	^ self defaultAttributes at: aName ifAbsent: nil! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:19'!
display
	^ self attributeAt: #display ! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 07:08'!
display: aString
	self attributeAt: #display put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:56'!
fill
	^ self inheritedAttributeAt: #fill ifAbsent: [ ASColorPaint new color: Color black ]! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:19'!
fill: aString
	self attributeAt: #fill put: (ASFillConverter new translate: aString trimBoth)! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 09:23'!
fill_opacity
	^ self inheritedAttributeAt: #fill_opacity ifAbsent: [ self opacity ]! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:19'!
fill_opacity: aString 
	self attributeAt: #fill_opacity  put: (self translateNumber: aString)! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 16:09'!
fill_rule: aString
	self attributeAt: #fill_rule put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:15'!
filter: aString 
	"<uri> | none | inherit"
	self attributeAt: #filter put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 15:57'!
font_size: aString 
	self attributeAt: #font_size put: aString! !

!ASElement methodsFor: 'accessing' stamp: 'IgorStasenko 4/27/2012 12:47'!
id
	^ id! !

!ASElement methodsFor: 'accessing' stamp: 'StephaneDucasse 12/6/2014 07:15'!
id: aString 
	id := aString.
	self registerId.! !

!ASElement methodsFor: 'attribute management' stamp: 'IgorStasenko 4/24/2012 14:53'!
ignoredAttribute
	"just a stub to indicate an ignored attribute"
! !

!ASElement methodsFor: 'attribute management' stamp: 'JochenRick 1/24/2014 09:30'!
inheritedAttributeAt: name
	^ attributes at: name ifAbsent: [ parent
		ifNil: [ self error: 'Attribute ', name asString, ' not found.' ]
		ifNotNil: [ parent inheritedAttributeAt: name ] ]! !

!ASElement methodsFor: 'attribute management' stamp: 'JochenRick 1/24/2014 09:25'!
inheritedAttributeAt: name ifAbsent: aBlock

	^ attributes at: name ifAbsent: [ parent
		ifNil: [ aBlock value ]
		ifNotNil: [ parent inheritedAttributeAt: name ifAbsent: aBlock ] ]! !

!ASElement methodsFor: 'initialize' stamp: 'IgorStasenko 4/25/2012 07:07'!
initialize 

	attributes := EmptyDict.

! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 17:10'!
marker: aString 
	self attributeAt: #marker put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 17:11'!
marker_mid: aString 
	self attributeAt: #marker_mid put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 17:10'!
marker_start: aByteString 
	self attributeAt: #marker_start put: aByteString ! !

!ASElement methodsFor: 'accessing' stamp: 'IgorStasenko 4/27/2012 11:50'!
noFill
	^ ASNoFill soleInstance ! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 09:21'!
opacity
	^ self inheritedAttributeAt: #opacity ifAbsent: 1! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 21:46'!
opacity: aString 
	self attributeAt: #opacity put: (self translateNumber: aString)! !

!ASElement methodsFor: 'accessing' stamp: 'IgorStasenko 4/7/2012 16:34'!
parent
	^ parent
! !

!ASElement methodsFor: 'accessing' stamp: 'IgorStasenko 4/7/2012 16:34'!
parent: anObject
	parent := anObject! !

!ASElement methodsFor: 'internal' stamp: 'StephaneDucasse 12/6/2014 07:22'!
registerId
	"register receiver in global id table"
	self root setId: id for: self! !

!ASElement methodsFor: 'rendering' stamp: 'StephaneDucasse 12/6/2014 07:15'!
renderOn: aCanvas
	"render on Athens canvas"

	self transform
		ifNotNil: [ :tr | 
			aCanvas pathTransform
				restoreAfter: [ 
					aCanvas pathTransform multiplyBy: tr.
					self renderShapeOn: aCanvas ] ]
		ifNil: [ self renderShapeOn: aCanvas ]! !

!ASElement methodsFor: 'rendering' stamp: 'IgorStasenko 4/25/2012 05:48'!
renderShapeOn: aCanvas
! !

!ASElement methodsFor: 'internal' stamp: 'StephaneDucasse 12/6/2014 07:25'!
resolveIdForAttribute: aName dict: dict
	| value |
	value := self perform: aName.
	value class == ASLink
		ifTrue: [ self attributeAt: aName put: (value resolveIds: dict) ]! !

!ASElement methodsFor: 'internal' stamp: 'StephaneDucasse 12/6/2014 07:22'!
resolveIds: idDict
	self resolveMyIds: idDict! !

!ASElement methodsFor: 'internal' stamp: 'StephaneDucasse 12/6/2014 07:25'!
resolveMyIds: idDict
	#(#fill #stroke #xlink_href) 
		do: [ :each | self resolveIdForAttribute: each dict: idDict ]! !

!ASElement methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 15:01'!
root

	^ parent ifNil: [ self ] ifNotNil: [ parent root ]! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:56'!
stroke
	^ self inheritedAttributeAt: #stroke ifAbsent: [ ASNoFill soleInstance ]! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 22:14'!
stroke: aString 
	self attributeAt: #stroke put:  (ASFillConverter new translate: aString trimBoth )! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:16'!
stroke_dasharray
	^ self inheritedAttributeAt: #stroke_dasharray ifAbsent: nil! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:19'!
stroke_dasharray: aString 
	self attributeAt: #stroke_dasharray put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:59'!
stroke_dashoffset
	^ self inheritedAttributeAt: #stroke_dashoffset ifAbsent: 0! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:19'!
stroke_dashoffset: aString 
	self attributeAt: #stroke_dashoffset put: (self translateLength: aString)! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:57'!
stroke_linecap
	^ self inheritedAttributeAt: #stroke_linecap ifAbsent: #butt! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 16:43'!
stroke_linecap: aString 
	self attributeAt: #stroke_linecap put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:57'!
stroke_linejoin
	^ self inheritedAttributeAt: #stroke_linejoin ifAbsent: #miter! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:23'!
stroke_linejoin: aString 
	self attributeAt: #stroke_linejoin put: aString! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:57'!
stroke_miterlimit
	^ self inheritedAttributeAt: #stroke_miterlimit ifAbsent: 4! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/27/2012 11:47'!
stroke_miterlimit: aString
	self attributeAt: #stroke_miterlimit put: (self translateNumber: aString)
! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:47'!
stroke_opacity
	^ self inheritedAttributeAt: #stroke_opacity ifAbsent: [ self opacity ]! !

!ASElement methodsFor: 'attributes' stamp: 'StephaneDucasse 12/6/2014 07:20'!
stroke_opacity: aString 
	self attributeAt: #stroke_opacity  put: (self translateNumber: aString)! !

!ASElement methodsFor: 'attributes' stamp: 'JochenRick 1/24/2014 10:57'!
stroke_width
	^ self inheritedAttributeAt: #stroke_width ifAbsent: 1! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 22:14'!
stroke_width: aString 
	self attributeAt: #stroke_width  put: (self translateNumber: aString)! !

!ASElement methodsFor: 'operations' stamp: 'IgorStasenko 4/27/2012 17:12'!
style: aString 
	| values |
	values := aString subStrings: ';'.
	
	values do: [:keyval | | key val setter |
		key := (keyval copyUpTo: $:) trimBoth.
		val := (keyval copyAfter: $:) trimBoth.
		setter := self attributeSetter: key.
		(self respondsTo: setter) ifTrue: [ self perform: setter with: val ]
		ifFalse: [ self attributeAt: setter allButLast asSymbol  put: val ]
		]
! !

!ASElement methodsFor: 'operations' stamp: 'IgorStasenko 4/25/2012 07:11'!
transform
	^ self attributeAt: #transform! !

!ASElement methodsFor: 'operations' stamp: 'StephaneDucasse 12/6/2014 07:16'!
transform: aString
	" rotate(10) scale (1) translate (100,10) "

	self attributeAt: #transform put: (ASTransformConverter new translateString: aString)! !

!ASElement methodsFor: 'translate' stamp: 'IgorStasenko 4/7/2012 14:41'!
translate: xmlNode
	self translateAttributes: xmlNode attributes.
! !

!ASElement methodsFor: 'translate' stamp: 'StephaneDucasse 12/6/2014 07:23'!
translate: aXMLElement parent: node 
	parent := node.
	^ self translate: aXMLElement! !

!ASElement methodsFor: 'translate' stamp: 'IgorStasenko 4/27/2012 16:06'!
translateAttributes: attrs
	attrs associationsDo: [:assoc |
		| setter |
		setter := self attributeSetter: assoc key.
		(self respondsTo: setter) ifTrue: [
			self perform: setter with: assoc value trimBoth ]
		ifFalse: [ self attributeAt: setter allButLast asSymbol put: assoc value trimBoth ]
	]! !

!ASElement methodsFor: 'translate' stamp: 'IgorStasenko 4/24/2012 14:58'!
translateColor: aString 
	"right now we do like that"
	^ Color fromString: aString! !

!ASElement methodsFor: 'translate' stamp: 'IgorStasenko 4/24/2012 16:04'!
translateLength: aString 

	^ self translateNumber: aString! !

!ASElement methodsFor: 'translate' stamp: 'IgorStasenko 4/23/2012 18:10'!
translateNumber: aString 

	^ (ASConverter new stream: aString readStream ) nextNumber 
! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/24/2012 14:53'!
xlink_actuate: aString 
	self ignoredAttribute ! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 22:28'!
xlink_href
	^ self attributeAt: #xlink_href! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/25/2012 22:24'!
xlink_href: aString 
	self attributeAt: #xlink_href put: (ASLink url: aString)! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/24/2012 14:54'!
xlink_show: aByteString 
	self ignoredAttribute ! !

!ASElement methodsFor: 'attributes' stamp: 'IgorStasenko 4/24/2012 14:52'!
xlink_type: aString 

	self ignoredAttribute! !

ASElement subclass: #ASFilter
	instanceVariableNames: 'x y width height'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASFilter methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 12/6/2014 07:31'!
color_interpolation_filters: aString 
	self attributeAt: #color_interpolation_filters put: aString! !

!ASFilter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 16:41'!
filterUnits: aString 
	self attributeAt: #filterUnits put: aString! !

!ASFilter methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:40'!
height: aString 
	height := self translateLength: aString! !

!ASFilter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 16:41'!
primitiveUnits: aString 
	self attributeAt: #primitiveUnits put: aString! !

!ASFilter methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:40'!
width: aString 
	width := self translateLength: aString! !

!ASFilter methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:39'!
x: aString 
	x := self translateLength: aString! !

!ASFilter methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2012 16:40'!
y: aString 
	y := self translateLength: aString! !

ASElement subclass: #ASGradientStop
	instanceVariableNames: 'color offset opacity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 14:48'!
asRampColorWithOpacity: aFloat 

		^ Association key: offset value: (color alpha: self opacity * aFloat)! !

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 10:50'!
offset
	^ offset! !

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:59'!
offset: aString 
	offset := self translateNumber: aString! !

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 14:44'!
opacity 
	^ opacity ifNil: [1]! !

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 17:52'!
setOpacity: number

	color := color alpha: number.
! !

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:57'!
stop_color: aString 

	color := self translateColor: aString! !

!ASGradientStop methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 15:14'!
stop_opacity: aString 

	opacity := self translateNumber: aString.
! !

ASElement subclass: #ASMetadata
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

ASElement subclass: #ASShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

ASShape subclass: #ASCircle
	instanceVariableNames: 'r cx cy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASCircle methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 16:05'!
cx: aString 
"cx = <coordinate>
The x-axis coordinate of the center of the circle.
If the attribute is not specified, the effect is as if a value of '0'  were specified."

	cx := self translateLength: aString! !

!ASCircle methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 16:06'!
cy: aString 

"cy = <coordinate>
The y-axis coordinate of the center of the circle.
If the attribute is not specified, the effect is as if a value of 0 were specified. Animatable: yes."

	cy := self translateLength: aString! !

!ASCircle methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 16:49'!
initialize 
	super initialize.
	
	cx := cy := 0.! !

!ASCircle methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 18:49'!
r: aString 

"r = <length>
The radius of the circle.
A negative value is an error (see Error processing). A value of zero disables rendering of the element.
Animatable: yes.
"

	r := self translateLength: aString.
	
! !

!ASCircle methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:58'!
setShapeOn: aCanvas
	| circle |

	circle := aCanvas createPath: [:builder |
		builder 
			absolute;
			moveTo: (cx - r @ cy);
			cwArcTo: (cx + r @ cy )  angle: 180 degreesToRadians;
			cwArcTo: (cx - r @ cy)  angle: 180 degreesToRadians
		].

	aCanvas setShape: circle.
! !

ASShape subclass: #ASEllipse
	instanceVariableNames: 'rx ry cx cy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASEllipse methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:20'!
cx: aString 
"cx = <coordinate>
The x-axis coordinate of the center of the circle.
If the attribute is not specified, the effect is as if a value of '0'  were specified."

	cx := self translateLength: aString! !

!ASEllipse methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:21'!
cy: aString 

"cy = <coordinate>
The y-axis coordinate of the center of the circle.
If the attribute is not specified, the effect is as if a value of 0 were specified. Animatable: yes."

	cy := self translateLength: aString! !

!ASEllipse methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:21'!
rx: aString 

	rx := self translateLength: aString.
	
! !

!ASEllipse methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:21'!
ry: aString 

	ry := self translateLength: aString.
	
! !

!ASEllipse methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:58'!
setShapeOn: aCanvas
	| circle |
	

	circle := aCanvas createPath: [:builder |
		builder 
			absolute;
			moveTo: (cx - rx @ cy);
			cwArcTo: (cx + rx @ cy )  angle: 180 degreesToRadians;
			cwArcTo: (cx - rx @ cy)  angle: 180 degreesToRadians
		].

	aCanvas setShape: circle.
! !

ASShape subclass: #ASLine
	instanceVariableNames: 'x1 y1 x2 y2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:48'!
setShapeOn: aCanvas
	| path |
	
	path := aCanvas createPath: [:builder |
		builder 
			absolute;
			moveTo: x1@y1;
			lineTo: x2@y2.
	].
	
	aCanvas setShape: path.
! !

!ASLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:11'!
x1: aString 
	x1 := self translateLength: aString! !

!ASLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:11'!
x2: aString 
	x2 := self translateLength: aString! !

!ASLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:11'!
y1: aString 
	y1 := self translateLength: aString! !

!ASLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:11'!
y2: aString 
	y2 := self translateLength: aString! !

ASShape subclass: #ASPath
	instanceVariableNames: 'pathData pathCache athensPath bbox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASPath methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 17:17'!
buildPathOn: aCanvas 

	pathCache := WeakArray with: aCanvas surface.
	athensPath := aCanvas createPath: [:builder |
		
		pathData do: [:each |
			builder perform: each first withArguments: each second
			]
	]
! !

!ASPath methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 17:15'!
checkCacheFor: aCanvas 

	pathCache ifNil: [ ^ self buildPathOn: aCanvas ].
	pathCache first == aCanvas surface ifFalse: [ ^ self buildPathOn: aCanvas ]! !

!ASPath methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 08:37'!
d: aString 

"	d = 'path data'
	A path is defined by including a 'path' element which contains a d=(path data) attribute, where the d attribute contains the moveto, 
	line, curve (both cubic and quadratic Béziers), arc and closepath instructions.
"

	| conv |
	conv := ASPathConverter new stream: aString readStream.

	pathData := conv convertPathData.
	bbox := conv bbox.! !

!ASPath methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 14:29'!
setShapeOn: aCanvas

	self checkCacheFor: aCanvas.
	aCanvas setShape: athensPath.
	! !

ASShape subclass: #ASPolygon
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

ASPolygon subclass: #ASPolyLine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASPolyLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 07:15'!
setShapeOn: aCanvas
	| path |

	path := aCanvas createPath: [:builder |
		builder 
			absolute;
			moveTo: points first.		

		points allButFirstDo: [:pt | builder lineTo: pt ].
		

	].
	
	aCanvas setShape: path.
! !

!ASPolygon methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/23/2012 16:00'!
points: aString 

	"parse polygon points"
	| pts x y conv stream |
	
	points := OrderedCollection new.
	stream := aString readStream.
	
	conv := ASConverter new stream: stream.

	[ stream atEnd ] whileFalse: [
		conv skipBlanks.
		x := conv nextNumber.
		x ifNil: [ ^ self ].
		
		conv skipBlanks.
		stream peek = $, ifFalse: [ ^ self].
		stream next.
		conv skipBlanks.
		y := conv nextNumber.
		y ifNil: [ ^ self ].
		
		points add: (x@y)
		
	].

	^ points
	! !

!ASPolygon methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:57'!
setShapeOn: aCanvas
	| path |

	path := aCanvas createPath: [:builder |
		builder 
			absolute;
			moveTo: points first.		

		points allButFirstDo: [:pt | builder lineTo: pt ].
		
		builder close
	].
	
	aCanvas setShape: path.
! !

ASShape subclass: #ASRect
	instanceVariableNames: 'x y width height rx ry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASRect commentStamp: '<historical>' prior: 0!
The 'rect' element defines a rectangle which is axis-aligned with the current user coordinate system. Rounded rectangles can be achieved by setting appropriate values for attributes rx and ry.

If a properly specified value is provided for rx but not for ry, then the user agent processes the 'rect' element with the effective value for ry as equal to rx. If a properly specified value is provided for ry but not for rx, then the user agent processes the 'rect' element with the effective value for rx as equal to ry. If neither rx nor ry has a properly specified value, then the user agent processes the 'rect' element as if no rounding had been specified, resulting in square corners. If rx is greater than half of the width of the rectangle, then the user agent processes the 'rect' element with the effective value for rx as half of the width of the rectangle. If ry is greater than half of the height of the rectangle, then the user agent processes the 'rect' element with the effective value for ry as half of the height of the rectangle.
Mathematically, a 'rect' element can be mapped to an equivalent 'path' element as follows: (Note: all coordinate and length values are first converted into user space coordinates according to Units.)
● perform an absolute moveto operation to location (x+rx,y), where x is the value of the 'rect' element's x attribute converted to user space, rx is the effective value of the rx attribute converted to user space and y is the value of the y attribute converted to user space
● perform an absolute horizontal lineto operation to location (x+width-rx,y), where width is the 'rect' element's width attribute converted to user space
● perform an absolute elliptical arc operation to coordinate (x+width,y+ry), where the effective values for the rx and ry attributes on the 'rect' element converted to user space are used as the rx and ry attributes on the elliptical arc command, respectively, the x-axis-rotation is set to zero, the large-arc-flag is set to zero, and the sweep-flag is set to one
● perform a absolute vertical lineto to location (x+width,y+height-ry), where height is the 'rect' element's height attribute converted to user space
● perform an absolute elliptical arc operation to coordinate (x+width-rx,y+height)
● perform an absolute horizontal lineto to location (x+rx,y+height)
● perform an absolute elliptical arc operation to coordinate (x,y+height-ry)
● perform an absolute absolute vertical lineto to location (x,y+ry)
● perform an absolute elliptical arc operation to coordinate (x+rx,y)!

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/23/2012 18:12'!
height: aString 

	height := self translateNumber: aString! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:42'!
initialize
	super initialize.
	
	x := y := rx := ry := 0.! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:55'!
rx: aString 

	rx := self translateNumber: aString! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:55'!
ry: aString 

	ry := self translateNumber: aString! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:56'!
setShapeOn: aCanvas
	aCanvas setShape: (Rectangle origin: x@y extent: width@height).

	"todo: take rx, ry into account"! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/23/2012 18:10'!
width: aString 

	width := self translateNumber: aString! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:54'!
x: aString 

	x := self translateNumber: aString! !

!ASRect methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:55'!
y: aString 

	y := self translateNumber: aString! !

!ASShape methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 12:56'!
renderShapeOn: aCanvas

"	(self id ='path3070-4' ) ifTrue: [ self halt ]."
	
	self display = 'none' ifTrue: [ ^ self ].
	
	self setShapeOn: aCanvas.
	self fill fill: self on: aCanvas.
	self stroke stroke: self on: aCanvas.
! !

!ASShape methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:55'!
setShapeOn: aCanvas
	self subclassResponsibility ! !

ASShape subclass: #ASText
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASText methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:14'!
font_family: aByteString 
	! !

!ASText methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:13'!
font_size: aByteString 
	! !

!ASText methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:15'!
setShapeOn: aCanvas
	! !

!ASText methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:13'!
x: aByteString 
	! !

!ASText methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:13'!
y: aByteString 
	! !

Object subclass: #ASLength
	instanceVariableNames: 'value units'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASLength commentStamp: '<historical>' prior: 0!
A coordinate or length value can be expressed as a number following by a unit identifier (e.g., "25cm" or "15em"). The list of unit identifiers in SVG matches the list of unit identifiers in CSS: em, ex, px, pt, pc, cm, mm, in and percentages. The following describes how the various unit identifiers are processed:!

!ASLength class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/23/2012 16:27'!
fromString: aString
	| conv l |
	conv := (ASConverter new stream: aString readStream).
	l := self new.
	l value:  conv nextNumber.
	l units: conv stream upToEnd.
	^ l! !

!ASLength methodsFor: 'printing' stamp: 'IgorStasenko 4/23/2012 16:28'!
printOn: aStream

	aStream print: value; nextPutAll: units! !

!ASLength methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:22'!
units
	^ units! !

!ASLength methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:22'!
units: anObject
	units := anObject! !

!ASLength methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:23'!
value
	^ value! !

!ASLength methodsFor: 'accessing' stamp: 'IgorStasenko 4/23/2012 16:22'!
value: anObject
	value := anObject! !

!ASLength methodsFor: 'converting' stamp: 'IgorStasenko 4/27/2012 15:57'!
valueForDPI: dpi


	units = 'cm' ifTrue: [
		^ value * (dpi / 2.54) asFloat
		].
	
	units = 'mm' ifTrue: [
		^ value * (dpi / 25.4) asFloat
		].

	units = 'px' ifTrue: [ ^ value ].

	units = 'pt' ifTrue: [ ^ value ].
	
	units isEmpty ifTrue: [ ^ value ].
	
	self halt.! !

Object subclass: #ASLink
	instanceVariableNames: 'url isId id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASLink class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 10:41'!
url: aString

	^ self new url: aString! !

!ASLink methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 15:57'!
id
	^ id! !

!ASLink methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 15:57'!
isId
	^ isId == true! !

!ASLink methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 16:43'!
resolveIds: dict 
	
	self isId ifTrue: [
		^ dict at: id ifAbsent: [ self error: 'unknown id: ', id ]
		].
	
	self error: 'cannot resolve url: ' , url
! !

!ASLink methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 15:54'!
url
	^ url! !

!ASLink methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 15:58'!
url: aString
	url := aString.
	
	isId := url first = $#.
	isId ifTrue: [ id := url allButFirst ]! !

Object subclass: #ASNoFill
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

ASNoFill class
	instanceVariableNames: 'soleInstance'!

ASNoFill class
	instanceVariableNames: 'soleInstance'!

!ASNoFill class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 16:46'!
soleInstance

	^ soleInstance ifNil: [ soleInstance  := self new ]
	! !

!ASNoFill methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 11:58'!
fill: aShape on: aCanvas
	"do nothing"! !

!ASNoFill methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 16:48'!
resolveIds: aDictionary 
	^ self! !

!ASNoFill methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 11:57'!
stroke: aShape on: aCanvas
	"do nothing" 
! !

Object subclass: #ASPaint
	instanceVariableNames: 'opacity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

ASPaint subclass: #ASColorPaint
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASColorPaint methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 17:45'!
color: aColor
	color := aColor! !

!ASColorPaint methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 11:57'!
fill: shape on: aCanvas

	aCanvas setPaint: (color alpha: (color alpha * shape fill_opacity)).
	
	aCanvas draw! !

!ASColorPaint methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 17:45'!
resolveIds: aDictionary 
	^ self ! !

!ASColorPaint methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 12:30'!
stroke: shape on: aCanvas
	| stroke |
	stroke := aCanvas setStrokePaint: (color alpha: (color alpha * shape stroke_opacity)).
	
	stroke width: shape stroke_width.
	
	aCanvas draw! !

Object subclass: #ASVGNodeClasses
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!ASVGNodeClasses class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 16:12'!
fromName: aNodeName
	(self canUnderstand: aNodeName asSymbol) ifTrue: [ ^ self new perform: aNodeName asSymbol ]
	ifFalse: [ ^ ASMetadata ]! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:55'!
circle
	^ ASCircle! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 19:18'!
defs
	^ ASDefs! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:10'!
desc
	^ ASMetadata! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 05:41'!
ellipse
	^ ASEllipse ! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 16:39'!
filter
	^ ASFilter! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/7/2012 16:23'!
g
	^ ASGroupElement! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:11'!
line
	^ ASLine! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 13:56'!
linearGradient
	^ ASLinearGradient! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 19:17'!
metadata
	^ ASMetadata! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/23/2012 18:13'!
path
	^ ASPath ! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/7/2012 16:24'!
polygon
	^ ASPolygon! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 07:16'!
polyline
	^ ASPolyLine ! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/25/2012 16:31'!
radialGradient
	^ ASRadialGradient! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/23/2012 18:08'!
rect
	^ ASRect! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/24/2012 14:56'!
stop
	
	^ ASGradientStop! !

!ASVGNodeClasses methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/27/2012 13:13'!
text
	^ ASText! !

Object subclass: #AthensSVGConverter
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-SVG'!

!AthensSVGConverter class methodsFor: 'instance creation' stamp: 'IgorStasenko 4/6/2012 17:03'!
fromFile: aFileName
	| doc |
	doc := XMLDOMParser parseFileNamed: aFileName.
	^ self new importXMLDocument: doc! !

!AthensSVGConverter class methodsFor: 'instance creation' stamp: 'IgorStasenko 4/6/2012 17:02'!
fromURL: anUrl
	| doc |
	doc := XMLDOMParser parse: anUrl asZnUrl retrieveContents asString.
	^ self new importXMLDocument: doc! !

!AthensSVGConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/7/2012 14:27'!
g
	"group"
	
	node attributes! !

!AthensSVGConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/7/2012 14:39'!
importXMLDocument: doc

	| root |
	root := doc nodes elements.
	
	((root size > 1) or: [ root first name ~= 'svg'])  ifTrue: [ self error: 'A root svg element expected' ].

	^ ASVGRoot new translate: root first.
! !

!AthensSVGConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/6/2012 17:29'!
visitNode: aNode
	| oldNode |
	oldNode := node.
	node := aNode.
	self perform: aNode name asSymbol.

	! !

ASElement initialize!
